function [survalue,survprob]=cnormtsurv(type,Y,d,mu,S,v,w,wcuts,j)

%cnormtsurv: survival values from a conditional t
%            with mean vector mu and covariance matrix S and v.
%
%----INPUT-----
% type:   1=log-normal,2=log-t
% d:      multivariate t degrees of freedom
% mu:     mean
% S,v:    (row and column) covariance matrices
% j:      element to condition upon, e.g. Y_j|Y_-j.
% w:      observed log survival time.
% wcuts:  values at which the conditional survival curve is evaluated
%
%----OUTPUT-----
% survalue: survival values at wcuts
%
%Copyright (c) 2006 Naijun Sha.
%**********************************************************************
  
n=length(Y);

Y1=Y(j);Y2=Y;Y2(j)=[];
mu1=mu(j);mu2=mu;mu2(j)=[];
S11=S(j,j);S12=S(j,:);S12(:,j)=[];S21=S12';
S22=S;S22(j,:)=[];S22(:,j)=[];
R=chol(S22);Rinv=inv(R);S22inv=Rinv*Rinv';%S22inv=inv(S22);
S112=S11-S12*S22inv*S21;
m1=mu1+S12*S22inv*(Y2-mu2);

a=S112;
b=v+(Y2-mu2)'*S22inv*(Y2-mu2);
c=sqrt(a*b/(d+n-1));
cuts=(wcuts-m1)/c;
points=(w'-m1)/c;
survalue=1-tcdf(cuts,d+n-1);
survprob=1-tcdf(points,d+n-1);
