function [MargGam,UniD,GammaD,PostGamD]=postinfer(...
    type,X,data,UniD,GammaD,What,lhat,alpha0,beta0,h0,Hg,s20,w,d)

%postinfer: computes posterior probs of visited models
%
%------INPUT--------
%apprtype:  1=log-normal,2=log-t
%X, data:   training data.
%GammaD:    visited models.
%What:      estimated "complete data" matrix.
%....
%
%-----OUTPUT--------
%PostGamD: log posterior probabilities of visited models (normalized).
%MargGam:   marginal posterior probabilities of inclusion of vars  
%     
%Copyright (c) 2006 Naijun Sha and Marina Vannucci.
%**********************************************************************


[n,p]=size(X);
T=data(:,1);     %---failure times----
dcen=data(:,2);  %---censored indicators----
Y=log(T);

%-----IG prior on sigma2
v0=3;
v2=v0*s20;

m=size(GammaD,1);
logprobnew=zeros(m,1);

for k=1:m
    r=str2num(GammaD{k});
    pr=length(r);
    Xr=X(:,r);
    beta0r=beta0(r);
    Hgr=Hg(r,r);
%---calculate the log posterior value of model r----
        switch type   %---distribution type----
           case 1        %---log-normal data---
             SIGr=eye(n)+h0*ones(n,n)+Xr*Hgr*Xr';
           case 2        %---log student-t---
              Lhat=diag(lhat);
              SIGr=d^2*Lhat+h0*ones(n,n)+Xr*Hgr*Xr';
       end
       %-----recompute log-posterior prob. log(f(r|...))------
         C=chol(SIGr);
         Cinv=inv(C);
         SIGrINV=Cinv*Cinv';%SIGrINV=inv(SIGr);
         switch length(w)
             case 1     
                 %----Bernoulli prior for variable index r---
                    logprob=pr*log(w)+(p-pr)*log(1-w)-1/2*log(det(SIGr))+v0/2*log(v2)-...
                         (n+v0)/2*log(v2+(What-alpha0*ones(n,1)-Xr*beta0r)'*SIGrINV*...
                         (What-alpha0*ones(n,1)-Xr*beta0r));
             case 2
                 %----Beta-Bernoulli prior for variable index r---
                    alpha=w(2)*w(1);	% (mean=pw(1), w(2)=alpha+beta) 
	                betapar=w(2)*(1-w(1));
                    logpostr=gammaln(alpha+pr)+gammaln(p+betapar-pr)-gammaln(alpha+betapar+p)...
                      -1/2*log(det(SIGr))+v0/2*log(v2)-...
                      (n+v0)/2*log(v2+(What-alpha0*ones(n,1)-Xr*beta0r)'*SIGrINV*...
                      (What-alpha0*ones(n,1)-Xr*beta0r));
         end
    logprobnew(k)=logprob;
end

%----posterior values of f(r|X,What)----
   logprobnew=logprobnew-mean(logprobnew);
   postr=exp(logprobnew);
   postr=postr/sum(postr);
   [spostr,index]=sort(-postr);
   GammaD=GammaD(index);
   PostGamD=-spostr;
   UniD=UniD(index);
   
%----marginal posterior of each variable selected----
   MargGam=marginale(GammaD,PostGamD,p);
   
