function [RSSval,MSEtr,MSEval,Tpred,Tvalpred,MargSurv,wcuts]=predsurv(...
    type,X,data,Xval,Tval,Gamma,PostGam,What,lhat,alpha0,beta0,h0,Hg,s20,d)

%predsurv: Prediction of survival times and survivor functions
%
%------INPUT--------
% type:                  1=log-normal,2=log-t
% X, data,Xval, Tval:    training and test data
% Gamma:                 visited models.
% PostGam:               posterior probability of visited models
% What:                  estimated "complete data" matrix.
% 
%-----OUTPUT--------
% wcuts:     grid points at which survival functions are evaluated.
% MargSurv:  survivor curves evaluated at wcuts
%
%
%Copyright (c) 2006 Naijun Sha and Marina Vannucci.
%**********************************************************************


T=data(:,1);     %---failure times----
dcen=data(:,2);  %---censoring indicators----
Y=log(T);

%-----IG prior on sigma2
v0=3;
v2=v0*s20;

[n,p]=size(X);
nval=size(Xval,1);
m=size(Gamma,1);

w=log(Tval);
stps=range(w)/100;
wcuts=[min(w)-stps:stps:max(w)+stps];

cutnum=length(wcuts);
MargSurv=zeros(nval,cutnum);
survpred=zeros(nval,nval);
Ypred=0;
Yvalpred=0;

%----prediction of survival curve----

disp(' ')
disp('------- Loop over visited models ...')
disp(' ')

for k=1:m 
    if k/500==round(k/500)
    disp(' ')
       	disp('Number of models:')
       	disp(k)
        disp('out of')
        disp(m)
       	disp('...')
    else
    end
    r=str2num(Gamma{k});
    Xr=X(:,r);
    pr=length(r);
    beta0r=beta0(r);
    Hgr=Hg(r,r);
       switch type   %---distribution type----
           case 1        %---log-normal data---
             aconst=1;
             SIGr=aconst*eye(n)+h0*ones(n,n)+Xr*Hgr*Xr';
           case 2        %---log t-student---
             Lhat=diag(lhat);
             SIGr=d^2*Lhat+h0*ones(n,n)+Xr*Hgr*Xr';
         end
    for j=1:nval
        xfr=Xval(j,r);Xfr=[xfr;Xr];Wfhat=[0;What];
        switch type 
          case 1    %----log-normal---
            muf=alpha0*ones(n+1,1)+Xfr*beta0r;
            SIGfr=aconst*eye(n+1)+h0*ones(n+1,n+1)+Xfr*Hgr*Xfr';
          case 2    %----log-t---
            muf=alpha0*ones(n+1,1)+Xfr*beta0r;
            Lhat=diag([0 lhat]);
            SIGfr=d^2*Lhat+h0*ones(n+1,n+1)+Xfr*Hgr*Xfr';
        end
        %----compute surv. curve at wcuts for jth subject in validation set---
        % input '1' since predicted covariates are 1st row of Xfr (see above) 
        [survalue,survprob]=cnormtsurv(type,Wfhat,v0,muf,SIGfr,v2,w,wcuts,1);
        MargSurv(j,:)=MargSurv(j,:)+survalue*PostGam(k);
    end

%----prediction of survival time for future obs Xf-----
     Xrstar=[ones(n,1) Xr];
     beta0rstar=[alpha0;beta0r];
     Hgrstar=[h0 zeros(1,pr);
     zeros(pr,1) Hgr];
     switch type 
       case 1   %---log-normal---
         Bhat=inv(1/aconst*Xrstar'*Xrstar+inv(Hgrstar))*...
             (1/aconst*Xrstar'*What+inv(Hgrstar)*beta0rstar);
       case 2   %---log-t---
       Lhat=diag(lhat);
       Bhat=inv(d^(-2)*Xrstar'*Lhat*Xrstar+inv(Hgrstar))*...
             (d^(-2)*Xrstar'*Lhat*What+inv(Hgrstar)*beta0rstar);
     end
     Yhat=Xrstar*Bhat;
     Xvalr=Xval(:,r);
     Xfrstar=[ones(nval,1) Xvalr];
     Yvalhat=Xfrstar*Bhat;

%---averaged survival time--- 
    Ypred=Ypred+Yhat*PostGam(k);
    Yvalpred=Yvalpred+Yvalhat*PostGam(k);

end   %----end of loop for visited models---


Tpred=exp(Ypred);                %---prediction of T----
Tvalpred=exp(Yvalpred);          %---prediction of Tval----
indexa=find(dcen);
MSEtr=sum((T(indexa)-Tpred(indexa)).^2)/length(T); %---mse for all exact survival time---
MSEval=sum((Tval-Tvalpred).^2)/length(Tval);     %---mse for all predicted survival time---
%---rss for "standarized" failure times----
RSSval=sum((Tval/max(Tval)-Tvalpred/max(Tvalpred)).^2);



